use std::collections::HashMap;

use material_design_2::MaterialDesign2Scheme;
use material_design_3::MaterialDesign3Scheme;
use wasm_bindgen::{prelude::wasm_bindgen, JsValue};

use crate::errors;

pub mod material_design_2;
pub mod material_design_3;

pub trait SchemeExport {
    fn output_css_variables(&self) -> String;
    fn output_scss_variables(&self) -> String;
    fn output_javascript_object(&self) -> String;
}

#[wasm_bindgen]
pub fn generate_material_design_3_scheme(
    source_color: &str,
    error_color: &str,
    custom_colors: JsValue,
) -> Result<JsValue, errors::ColorError> {
    let custom_colors: HashMap<String, String> = serde_wasm_bindgen::from_value(custom_colors)
        .map_err(|_| errors::ColorError::UnableToParseArgument)?;
    let mut scheme = MaterialDesign3Scheme::new(source_color, error_color)?;
    for (name, color) in custom_colors {
        scheme.add_custom_color(name, color)?;
    }
    Ok(serde_wasm_bindgen::to_value(&(
        scheme.clone(),
        scheme.output_css_variables(),
        scheme.output_scss_variables(),
        scheme.output_javascript_object(),
    ))
    .map_err(|_| errors::ColorError::UnableToAssembleOutput)?)
}

#[wasm_bindgen]
pub fn generate_material_design_2_scheme(
    primary_color: &str,
    secondary_color: &str,
    error_color: &str,
    custom_colors: JsValue,
) -> Result<JsValue, errors::ColorError> {
    let custom_colors: HashMap<String, String> = serde_wasm_bindgen::from_value(custom_colors)
        .map_err(|_| errors::ColorError::UnableToParseArgument)?;
    let mut scheme = MaterialDesign2Scheme::new(primary_color, secondary_color, error_color)?;
    for (name, color) in custom_colors {
        scheme.add_custom_color(&name, &color)?;
    }
    Ok(serde_wasm_bindgen::to_value(&(
        scheme.clone(),
        scheme.output_css_variables(),
        scheme.output_scss_variables(),
        scheme.output_javascript_object(),
    ))
    .map_err(|_| errors::ColorError::UnableToAssembleOutput)?)
}
